-- DATABASE STRUCTURE CONVERSION FROM 5.5 (Revision 1) TO 5.7 (Revision 2) --

-- Make changes from pre-released revision (5.5.9-5.5.11) to Revision 1:
ALTER TABLE ic_systemsettings MODIFY COLUMN paramval LONGTEXT;
ALTER TABLE ic_groups MODIFY COLUMN groupname VARCHAR(255);
ALTER TABLE ic_messagetypes MODIFY COLUMN typename VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN userlogin VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN userpassword VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN username VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN timezone VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN locale VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN email VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN phonelocal VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN phoneext VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN phonemob VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN fax VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN userposition VARCHAR(255);

-- truncate big user pictures
UPDATE ic_users SET picture=NULL WHERE picture IS NOT NULL AND length(picture)>65535;
ALTER TABLE ic_users MODIFY COLUMN picture TEXT;

ALTER TABLE ic_users MODIFY COLUMN ipint VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN ipext VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN hostname VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN dnsname VARCHAR(255);
ALTER TABLE ic_users MODIFY COLUMN clientversion VARCHAR(255);
ALTER TABLE ic_files MODIFY COLUMN savedto TEXT;
ALTER TABLE ic_recipientlists MODIFY COLUMN listname VARCHAR(255);

-- Extend database structure:
ALTER TABLE ic_recipients ADD COLUMN processed BOOLEAN NOT NULL DEFAULT FALSE;
CREATE INDEX ic_recipients_processed_index ON ic_recipients (processed);
ALTER TABLE ic_recipients ADD COLUMN pcomment TEXT;
ALTER TABLE ic_recipients ADD COLUMN pdate BIGINT;

-- For every non-file row of ic_messages set processed=TRUE and pdate=mdate for 
-- all the messages with id<lastreadmsgid. This query could be VERY long in time!
UPDATE 
ic_recipients, ic_users, ic_messages
SET
ic_recipients.processed=TRUE, 
ic_recipients.pdate=ic_messages.mdate 
WHERE 
ic_recipients.rid=ic_users.id AND 
ic_recipients.mid<=ic_users.lastreadmsgid AND 
ic_recipients.mid=ic_messages.id AND 
ic_messages.tid<>(SELECT id FROM ic_messagetypes WHERE templatename='IC_FILES' LIMIT 1);

-- Copy data about processed files from ic_files to ic_recipients:
UPDATE 
ic_recipients, ic_files 
SET 
ic_recipients.processed=TRUE, 
ic_recipients.pdate=ic_files.fdate, 
ic_recipients.pcomment=ic_files.savedto 
WHERE 
ic_recipients.mid=ic_files.mid AND 
ic_recipients.rid=ic_files.rid;

-- Remove unneeded data:
DROP TABLE ic_files;
ALTER TABLE ic_users DROP COLUMN lastreadmsgid;

-- Add new necessary tables:
CREATE TABLE ic_ldap (
    id INTEGER UNIQUE NOT NULL,
    ldapdomain VARCHAR(255) UNIQUE NOT NULL,
    ldapurl VARCHAR(255) NOT NULL,
    ldapprincipalformat VARCHAR(255) NOT NULL,
    ldapcnattribute VARCHAR(255) NOT NULL,
    ldaploginattribute VARCHAR(255) NOT NULL,
    ldapmappingattribute VARCHAR(255) NOT NULL,
    ldapisbinaryattribute BOOLEAN NOT NULL DEFAULT FALSE,
    ldapsearchbase VARCHAR(255) NOT NULL,
    ldapfilter VARCHAR(255) NOT NULL,
    PRIMARY KEY (id)
) ENGINE=InnoDB;

CREATE TABLE ic_auth_internal (
    uid INTEGER UNIQUE NOT NULL REFERENCES ic_users (id) ON DELETE CASCADE ON UPDATE CASCADE,
    userlogin VARCHAR(255) UNIQUE NOT NULL,
    userpassword VARCHAR(255) NOT NULL,
    PRIMARY KEY (userlogin)
) ENGINE=InnoDB;

CREATE TABLE ic_auth_external (
    uid INTEGER UNIQUE NOT NULL REFERENCES ic_users (id) ON DELETE CASCADE ON UPDATE CASCADE,
    ldapmapping VARCHAR(255) NOT NULL,
    lid INTEGER NOT NULL REFERENCES ic_ldap (id),
    PRIMARY KEY (ldapmapping, lid)
) ENGINE=InnoDB;

CREATE TABLE ic_messages_binarydata (
    mid BIGINT NOT NULL REFERENCES ic_messages (id),
    chunk_id BIGINT NOT NULL,
    chunk_data BLOB NOT NULL,
    PRIMARY KEY (mid, chunk_id)
) ENGINE=InnoDB;

CREATE INDEX ic_messages_binarydata_mid_index ON ic_messages_binarydata (mid);

CREATE TABLE ic_parameters (
    uid INTEGER NOT NULL REFERENCES ic_users (id) ON UPDATE CASCADE ON DELETE CASCADE,
    paramname VARCHAR(255) NOT NULL,
    paramval LONGTEXT NOT NULL,
    PRIMARY KEY (uid, paramname)
) ENGINE=InnoDB;

-- Copy existing user data from ic_users to ic_auth_internal:
INSERT INTO ic_auth_internal (uid, userlogin, userpassword) SELECT id, userlogin, userpassword FROM ic_users;

-- Remove unneeded data:
ALTER TABLE ic_users DROP COLUMN userlogin;
ALTER TABLE ic_users DROP COLUMN userpassword;

-- DATABASE STRUCTURE CONVERTION IS FINISHED!!! --
